/*
 *  Copyright (c) 2003 Helsinki Institute for Information Technology
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to
 *  deal in the Software without restriction, including without limitation the
 *  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 *  sell copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 *  IN THE SOFTWARE.
 */

package seco.util.iterator;

import com.hp.hpl.jena.util.iterator.ClosableIterator;
import java.util.*;
/**
 *  Description of the Class
 *
 * @author     Samppa
 * @created    27. elokuuta 2003
 */
public class BasicIterator implements ClosableIterator {

	/**
	 *  Description of the Field
	 */
	private Iterator andThen        = null;

	/**
	 *  Description of the Field
	 */
	private IterationFilter filter  =
		new IterationFilter() {
			public boolean accept(Object o) {
				return true;
			}
		};
	/**
	 *  Description of the Field
	 */
	private Iterator iter           = null;
	/**
	 *  Description of the Field
	 */
	private IterationMap map        =
		new IterationMap() {
			public Object map(Object o) {
				return o;
			}
		};
	/**
	 *  Description of the Field
	 */
	private Object next             = null;


	/**
	 *  Constructor for the BasicIterator object
	 */
	public BasicIterator() { }


	/**
	 *  Constructor for the BasicIterator object
	 *
	 * @param  singleton  Description of the Parameter
	 */
	public BasicIterator(Object singleton) {
		this.next = singleton;
	}


	/**
	 *  Constructor for the BasicIterator object
	 *
	 * @param  iter  Description of the Parameter
	 */
	public BasicIterator(Iterator iter) {
		this(iter, null, null);
	}


	/**
	 *  Constructor for the BasicIterator object
	 *
	 * @param  iter    Description of the Parameter
	 * @param  filter  Description of the Parameter
	 */
	public BasicIterator(Iterator iter, IterationFilter filter) {
		this(iter, filter, null);
	}


	/**
	 *  Constructor for the BasicIterator object
	 *
	 * @param  iter    Description of the Parameter
	 * @param  filter  Description of the Parameter
	 * @param  map     Description of the Parameter
	 */
	public BasicIterator(Iterator iter, IterationFilter filter, IterationMap map) {
		this.iter = iter;
		if (filter != null) {
			this.filter = filter;
		}
		if (map != null) {
			this.map = map;
		}
	}


	/**
	 *  Constructor for the BasicIterator object
	 *
	 * @param  iter  Description of the Parameter
	 * @param  map   Description of the Parameter
	 */
	public BasicIterator(Iterator iter, IterationMap map) {
		this(iter, null, map);
	}


	/**
	 *  Constructor for the BasicIterator object
	 *
	 * @param  iter     Description of the Parameter
	 * @param  andThen  Description of the Parameter
	 */
	public BasicIterator(BasicIterator iter, Iterator andThen) {
		this(iter, iter.filter, iter.map);
		this.andThen = andThen;
	}


	/**
	 *  Description of the Method
	 *
	 * @param  andThen  Description of the Parameter
	 * @return          Description of the Return Value
	 */
	public BasicIterator andThen(Iterator andThen) {
		return new BasicIterator(this, andThen);
	}


	/**
	 *  Description of the Method
	 */
	public void close() {
		if (this.iter instanceof ClosableIterator) {
			((ClosableIterator)this.iter).close();
		}
		this.iter = null;
		this.next = null;
		if (this.andThen != null) {
			if (this.andThen instanceof ClosableIterator) {
				((ClosableIterator)this.andThen).close();
			}
			this.andThen = null;
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @return    Description of the Return Value
	 */
	public boolean hasNext() {
		boolean rv  = false;
		while (this.next == null && this.iter != null && iter.hasNext()) {
			this.next = iter.next();
			if (!filter.accept(this.next)) {
				this.next = null;
			}
		}
		rv = (this.next != null || (this.andThen != null && this.andThen.hasNext()));
		if (!rv) {
			close();
		}
		return rv;
	}


	/**
	 *  Description of the Method
	 *
	 * @return    Description of the Return Value
	 */
	public Object next() {
		Object rv  = null;
		if (hasNext()) {
			if (this.next != null) {
				rv = this.next;
				this.next = null;
			} else {
				rv = this.andThen.next();
			}
			rv = this.map.map(rv);
		} else {
			throw new NoSuchElementException();
		}
		return rv;
	}


	/**
	 *  Description of the Method
	 */
	public void remove() {
		throw new UnsupportedOperationException();
	}


	/**
	 *  Description of the Method
	 *
	 * @param  iter  Description of the Parameter
	 * @return       Description of the Return Value
	 */
	public static BasicIterator uniqueIterator(Iterator iter) {
		return new BasicIterator(iter, new UniqueFilter());
	}
}

